@@ -24,6 +24,7 @@
 #include <netdb.h>
 #include <signal.h>
 #include <sys/uio.h>
+#include <sys/types.h>
 #include <sys/time.h>
 #include <sys/wait.h>
 #include <sys/socket.h>
@@ -36,6 +37,15 @@
 
 #include "cache.h"
 
+#ifndef MAP_LOCKED
+#define MAP_LOCKED 0
+#endif
+
+#if !defined(MAP_ANONYMOUS) && defined(MAP_ANON)
+#define MAP_ANONYMOUS MAP_ANON
+#endif
+
+
 #if FUSE_VERSION >= 23
 #define SSHFS_USE_INIT
 #endif
@@ -278,6 +288,7 @@ enum {
 	KEY_HELP,
 	KEY_VERSION,
 	KEY_FOREGROUND,
+	KEY_CONFIGFILE,
 };
 
 #define SSHFS_OPT(t, p, v) { t, offsetof(struct sshfs, p), v }
@@ -311,6 +322,7 @@ static struct fuse_opt sshfs_opts[] = {
 	FUSE_OPT_KEY("debug",          KEY_FOREGROUND),
 	FUSE_OPT_KEY("-d",             KEY_FOREGROUND),
 	FUSE_OPT_KEY("-f",             KEY_FOREGROUND),
+	FUSE_OPT_KEY("-F ",            KEY_CONFIGFILE),
 	FUSE_OPT_END
 };
 
@@ -650,29 +662,29 @@ static int buf_get_attrs(struct buffer *buf, struct stat *stbuf, int *flagsp)
 
 static int buf_get_statvfs(struct buffer *buf, struct statvfs *stbuf)
 {
-	uint32_t bsize;
-	uint32_t frsize;
+	uint64_t bsize;
+	uint64_t frsize;
 	uint64_t blocks;
 	uint64_t bfree;
 	uint64_t bavail;
 	uint64_t files;
 	uint64_t ffree;
 	uint64_t favail;
-	uint32_t fsid;
-	uint32_t flag;
-	uint32_t namemax;
+	uint64_t fsid;
+	uint64_t flag;
+	uint64_t namemax;
 
-	if (buf_get_uint32(buf, &bsize) == -1 ||
-	    buf_get_uint32(buf, &frsize) == -1 ||
+	if (buf_get_uint64(buf, &bsize) == -1 ||
+	    buf_get_uint64(buf, &frsize) == -1 ||
 	    buf_get_uint64(buf, &blocks) == -1 ||
 	    buf_get_uint64(buf, &bfree) == -1 ||
 	    buf_get_uint64(buf, &bavail) == -1 ||
 	    buf_get_uint64(buf, &files) == -1 ||
 	    buf_get_uint64(buf, &ffree) == -1 ||
 	    buf_get_uint64(buf, &favail) == -1 ||
-	    buf_get_uint32(buf, &fsid) == -1 ||
-	    buf_get_uint32(buf, &flag) == -1 ||
-	    buf_get_uint32(buf, &namemax) == -1) {
+	    buf_get_uint64(buf, &fsid) == -1 ||
+	    buf_get_uint64(buf, &flag) == -1 ||
+	    buf_get_uint64(buf, &namemax) == -1) {
 		return -1;
 	}
 
@@ -1338,11 +1350,13 @@ static int sftp_init_reply_ok(struct buffer *buf, uint32_t *version)
 
 			DEBUG("Extension: %s <%s>\n", ext, extdata);
 
-			if (strcmp(ext, SFTP_EXT_POSIX_RENAME) == 0) {
+			if (strcmp(ext, SFTP_EXT_POSIX_RENAME) == 0 &&
+			    strcmp(extdata, "1") == 0) {
 				sshfs.ext_posix_rename = 1;
 				sshfs.rename_workaround = 0;
 			}
-			if (strcmp(ext, SFTP_EXT_STATVFS) == 0)
+			if (strcmp(ext, SFTP_EXT_STATVFS) == 0 &&
+			    strcmp(extdata, "2") == 0)
 				sshfs.ext_statvfs = 1;
 		} while (buf2.len < buf2.size);
 	}
@@ -2116,6 +2130,7 @@ static int sshfs_open_common(const char *path, mode_t mode,
 	uint32_t pflags = 0;
 	struct iovec iov;
 	uint8_t type;
+	uint64_t wrctr = cache_get_write_ctr();
 
 	if ((fi->flags & O_ACCMODE) == O_RDONLY)
 		pflags = SSH_FXF_READ;
@@ -2171,7 +2186,7 @@ static int sshfs_open_common(const char *path, mode_t mode,
 	}
 
 	if (!err) {
-		cache_add_attr(path, &stbuf);
+		cache_add_attr(path, &stbuf, wrctr);
 		buf_finish(&sf->handle);
 		fi->fh = (unsigned long) sf;
 	} else {
@@ -2823,6 +2838,7 @@ static void usage(const char *progname)
 "SSHFS options:\n"
 "    -p PORT                equivalent to '-o port=PORT'\n"
 "    -C                     equivalent to '-o compression=yes'\n"
+"    -F ssh_configfile      specifies alternative ssh configuration file\n"
 "    -1                     equivalent to '-o ssh_protocol=1'\n"
 "    -o reconnect           reconnect to server\n"
 "    -o sshfs_sync          synchronous writes\n"
@@ -2913,6 +2929,12 @@ static int sshfs_opt_proc(void *data, const char *arg, int key,
 		ssh_add_arg("-oCompression=yes");
 		return 0;
 
+	case KEY_CONFIGFILE:
+		tmp = g_strdup_printf("-F%s", arg + 2);
+		ssh_add_arg(tmp);
+		g_free(tmp);
+		return 0;
+
 	case KEY_HELP:
 		usage(outargs->argv[0]);
 		fuse_opt_add_arg(outargs, "-ho");
